﻿-- Written by : Rakreo of Duskwood

-- Change to what chatframe output is desired
local chatFrame= _G['ChatFrame'..1];

-- Define colors used in this addon
local color = {['yellow'] = 'ffff78', ['orange'] = 'ff7831'};

-- Build translation table L
local locale, L = GetLocale(), {};
-- Translations for the English client
L['Max Level'] = 'Max Level';
L['Level'] = 'Level ';
L['XP'] = 'XP';
L['Beginning of'] = 'Beginning of ';
L['End of'] = 'End of ';
L['Max Level Guild'] = 'Max Level Guild';
L['Guild'] = ' Guild';
L['Guild XP'] = 'Guild XP';
L['Repetitions'] = 'Repetitions';
L['Standing'] = 'New standing with |cff%s%s|r is |cff%s%s|r!';
-- Translations for the French client
if (locale == 'frFR') then
-- Translations for the German client
elseif (locale == 'deDE') then
-- Translations for the Korean client
elseif (locale == 'koKR') then
-- Translations for the Chinese (simplified) client
elseif (locale == 'zhCN') then
-- Translations for the Chinese (traditional) client
elseif (locale == 'zhTW') then
-- Translations for the Russian client
elseif (locale == 'ruRU') then
-- Translations for the Spanish (Spain) client
elseif (locale == 'esES') then
-- Translations for the Spanish (Mexico) client
elseif (locale == 'esMX') then
end;
----------------------------------------------------------
-- Create the Frame and Register all the Events we need --
----------------------------------------------------------
-- Create a Frame for this addon so we can attach scripts and define the array for the functions that need to be registered
ReXP = CreateFrame('Frame'); local events = {};
---------------------------------------------------------------------
-- Fired after the Saved Variables for this addon have been loaded --
---------------------------------------------------------------------
function events:ADDON_LOADED(...)
	-- If the Addon loaded was ReXP then process it
	if ... == 'ReXP' then
		-- Make sure the database has been populated with the defaults
		ReXP:InitializeDB();
	end;
end;
-----------------------------------------------------------------------------------------------------------------------------
-- Fired when the games first loads or when entering/leaving an instance (Basically anytime the 'Loading' screen is shown) --
-----------------------------------------------------------------------------------------------------------------------------
function events:PLAYER_ENTERING_WORLD(...)
	-- Scan the Players XP and store it in the DB
	ReXP:ScanXP();
	-- Scan the Guild's XP and store it in the DB
	ReXP:ScanGXP();
	-- Scan the Players Reputation and store it in the DB
	if (GetNumFactions() == 0) then DEFAULT_CHAT_FRAME:AddMessage('Factions not yet available.') end;
	-- Expand all headers
	ExpandAllFactionHeaders();
	-- Loop thru all the factions
	for i = 1, GetNumFactions() do
		-- Fetch the details for this Faction
		local name, _, standingID, _, _, barValue, _, _, isHeader, _, hasRep = GetFactionInfo(i);
		-- If this is a Faction then show any changes
		if (not isHeader or hasRep) then
			-- Save the data for later
			ReXPDB['Rep'][name] = {['Standing'] = standingID, ['Value'] = barValue};
		end;
	end;
end;
------------------------------------------------------------------------------
-- Fired when the player's XP is updated (due quest completion or killing). --
------------------------------------------------------------------------------
function events:PLAYER_XP_UPDATE(...)
	-- If the person gaining XP is us then process it
	if ... == 'player' then
		-- Scan the Players XP and show any changes
		ReXP:ScanXP();
	end;
end;
-------------------------------------------
-- Fired when the Guild's XP is updated. --
-------------------------------------------
function events:GUILD_XP_UPDATE()
	-- This should not be needed, but, better safe than sorry
	if IsInGuild() == 1 then
		-- Scan the Guild XP and show any changes
		ReXP:ScanGXP();
	end;
end;
---------------------------------------------------
-- Fired when the Players Reputation is updated. --
---------------------------------------------------
function events:UPDATE_FACTION()
	-- Scan the Players Factions and show any changes
	ReXP:ScanRep();
end;
-------------------------------------------------------------------------------------------------------
-- Set the 'OnEvent' trigger assigned to ReXP and execute the function based on the Event that fired --
-------------------------------------------------------------------------------------------------------
ReXP:SetScript('OnEvent', function(self, event, ...) events[event](self, ...); end);
-- Loop thru all the 'events' array and register the 'Events'
for k, v in pairs(events) do
	-- Register all events for which handlers have been defined
	ReXP:RegisterEvent(k);
end;
-----------------------------------------------------------
-- Function to make sure the DataBase has default values --
-----------------------------------------------------------
function ReXP:InitializeDB()
	-- Build ReXPDB so we can use it later
	ReXPDB = {['XP'] = 0, ['GXP'] = 0, ['Rep'] = {}};
end;
-------------------------------------------------------------------------------
-- Function to scan Players XP, Reputation and Guild XP and save the values. --
-------------------------------------------------------------------------------
function ReXP:ScanXP()
	-- If we detected an XP gain then show the details
	if ReXPDB['XP'] > 0 and UnitXP('player') > ReXPDB['XP'] then
		-- Return the total amount of experience points required for the player to gain a level
		local playerMaxXP = UnitXPMax('player');
		-- Calculate the amount of XP gained
		local gain = (UnitXP('player') - ReXPDB['XP']);
		-- Calculate the amount of XP remaining until the next level
		local remain = playerMaxXP - UnitXP('player');
		-- Returns a unit's level. Returns -1 for boss units and hostile units whose level is ten levels or more above the player's.
		local level = UnitLevel('player');
		-- Returns the maximum level attainable by a player
		local maxLevel = GetMaxPlayerLevel();
		-- Calculate the next Level text
		local levelText; if (level + 1) == maxLevel then levelText = L['Max Level'] else levelText = L['Level']..(level + 1) end;
		-- Calculate the number of repetitions needed for next level
		local repetitions = ceil(remain / gain);
		-- Build the pretty XP gain message
		ReXP:ShowGain(gain, L['XP'], remain, levelText, repetitions);
	end;
	-- Update the database with the new XP
	ReXPDB['XP'] = UnitXP('player');
end;
--------------------------------------------------------------
-- Function to scan Players Reputation and save the values. --
--------------------------------------------------------------
function ReXP:ScanRep()
	-- Make sure we can see the Factions
	if (GetNumFactions() == 0) then DEFAULT_CHAT_FRAME:AddMessage('Factions not yet available.') end;
	-- Loop thru all the factions
	for i = 1, GetNumFactions() do
		-- Fetch the details for this Faction
		local name, _, standingID, barMin, barMax, barValue, _, _, isHeader, _, hasRep = GetFactionInfo(i);
		-- If this is a Faction then show any changes
		if (not isHeader or hasRep) then
			-- See if we have seen this Faction before
			if ReXPDB['Rep'][name] then
				-- See if the Standing has changed
				if ReXPDB['Rep'][name]['Standing'] ~= standingID then
					-- Build the pretty XP gain message
					local newstandingtext = string.format(L['Standing'], color['yellow'], name, color['yellow'], _G['FACTION_STANDING_LABEL'..standingID]);
					-- Send the new message the the ChatFrame selected
					chatFrame:AddMessage(newstandingtext);
				-- See if the reputation has changed with this Faction
				elseif ReXPDB['Rep'][name]['Value'] ~= barValue then
					-- Calculate the gain/loss
					local gain = barValue - ReXPDB['Rep'][name]['Value'];
					-- Calculate the amount of Reputation remaining until the next level
					local remain, levelText;
					if gain < 0 then
						remain = barMin - barValue;
						-- Calculate the next Level text
						if standingID > 1 then levelText = _G['FACTION_STANDING_LABEL'..standingID - 1] else levelText = L['Beginning of'].._G['FACTION_STANDING_LABEL'..1] end;
					else
						remain = barMax - barValue;
						-- Calculate the next Level text
						if standingID < 8 then levelText = _G['FACTION_STANDING_LABEL'..standingID + 1] else levelText = L['End of'].._G['FACTION_STANDING_LABEL'..8] end;
					end;
					-- Calculate the number of repetitions needed for next level
					local repetitions = ceil(remain / gain);
					-- Build the pretty XP gain message
					ReXP:ShowGain(gain, name, remain, levelText, repetitions);
				end;
			else
				-- Build the pretty XP gain message
				local newstandingtext = string.format(L['Standing'], color['yellow'], name, color['yellow'], _G['FACTION_STANDING_LABEL'..standingID]);
				-- Send the new message the the ChatFrame selected
				chatFrame:AddMessage(newstandingtext);
			end;
			-- Save the data for later
			ReXPDB['Rep'][name] = {['Standing'] = standingID, ['Value'] = barValue}; 
		end;
	end;
end;
----------------------------------------------------
-- Function to scan Guild XP and save the values. --
----------------------------------------------------
function ReXP:ScanGXP()
	-- Capture the XP stats for our Guild
	local currentXP, nextLevelXP = UnitGetGuildXP('player');
	-- If there has been a change since last time then show it
	if ReXPDB['GXP'] > 0 and currentXP > ReXPDB['GXP'] then
		-- Calculate the amount of XP gained
		local gain = (currentXP - ReXPDB['GXP']);
		-- Calculate the amount of XP remaining until the next level
		local remain = nextLevelXP;
		-- Returns the guild level as displayed in the guild pane. In World of Warcraft: Cataclysm, this level may be between 1 and 25, both inclusive.
		local level = GetGuildLevel();
		-- Calculate the next Level text
		local levelText; if (level + 1) == 25 then levelText = L['Max Level Guild'] else levelText = L['Level']..(level + 1)..L['Guild'] end;
		-- Calculate the number of repetitions needed for next level
		local repetitions = ceil(remain / gain);
		-- Build the pretty XP gain message
		ReXP:ShowGain(gain, L['Guild XP'], remain, levelText, repetitions);
	end;
	-- Update the database with the new XP
	ReXPDB['GXP'] = currentXP;
end;
------------------------------------------------------
-- Function to show the changes to a players stats. --
------------------------------------------------------
function ReXP:ShowGain(gain, name, remain, levelText, repetitions)
	-- Build the pretty XP gain message
	local newvaluetext = string.format('|cff%s%+d|cff%s %s|r, |cff%s%d|r more to |cff%s%s|r (|cff%s%d|r '..L['Repetitions']..').', color['orange'], gain, color['yellow'], name, color['orange'], remain, color['yellow'], levelText, color['orange'], repetitions);
	-- Send the new message the the ChatFrame selected
	chatFrame:AddMessage(newvaluetext);
end;-------------------------------------------------------------
-- Function to out Default XP and Reputation Chat messages --
-------------------------------------------------------------
local function ReXPFilter(self, event, msg, ...)
	-- Send the original messages into the bit bucket
	return true;
end;
-- Hook into the "CHAT_MSG_???" events so we can process them
ChatFrame_AddMessageEventFilter('CHAT_MSG_COMBAT_GUILD_XP_GAIN', ReXPFilter);
ChatFrame_AddMessageEventFilter('CHAT_MSG_COMBAT_XP_GAIN', ReXPFilter);
ChatFrame_AddMessageEventFilter('CHAT_MSG_COMBAT_FACTION_CHANGE', ReXPFilter);
